
{*******************************************************}
{                                                       }
{       RichView                                        }
{       This demo draws the first page of RTF document  }
{       in metafile (with header and footer, if         }
{       they are present).                              }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}

{
  v3.0: trackbar allows changing page
  v2.0: you can choose between screen and printer resolution
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, RVStyle, Menus, ExtCtrls, PtblRV, RVReport, ComCtrls;

type

  TPageInfo = record
    PageSize: TSize;
    Doc, Header, Footer: TRect;
  end;

  TForm1 = class(TForm)
    rvh: TRVReportHelper;
    rvhHeader: TRVReportHelper;
    rvhFooter: TRVReportHelper;
    Image1: TImage;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    mitOpen: TMenuItem;
    OpenDialog1: TOpenDialog;
    RVStyle1: TRVStyle;
    mitSave: TMenuItem;
    SaveDialog1: TSaveDialog;
    Options1: TMenuItem;
    mitScreen: TMenuItem;
    mitPrinter: TMenuItem;
    Panel1: TPanel;
    tbPages: TTrackBar;
    View1: TMenuItem;
    mitAutoResize: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure mitOpenClick(Sender: TObject);
    procedure mitSaveClick(Sender: TObject);
    procedure mitScreenClick(Sender: TObject);
    procedure mitPrinterClick(Sender: TObject);
    procedure tbPagesChange(Sender: TObject);
    procedure mitAutoResizeClick(Sender: TObject);
  private
    { Private declarations }
    MarginsMM: TRect;
    HeaderYMM, FooterYMM: Integer;
    PageSizeMM: TSize;
    UsePrinterResolution, Loaded: Boolean;
    PageInfo: TPageInfo;
    procedure InitHelper(h: TRVReportHelper);
    procedure CreateMetafile(var wmf: TMetafile; var Canvas: TMetafileCanvas);
    procedure FillPageInfo;
    procedure MakePageImage(PageNo: Integer);
    procedure DrawPage(Canvas: TCanvas; PageNo: Integer; const PageInfo: TPageInfo);
    procedure CalcPageLayout(Canvas: TCanvas; var PageInfo: TPageInfo);
  public
    { Public declarations }

  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.InitHelper(h: TRVReportHelper);
begin
  h.RichView.Style := RVStyle1;
  h.RichView.RTFReadProperties.TextStyleMode := rvrsAddIfNeeded;
  h.RichView.RTFReadProperties.ParaStyleMode := rvrsAddIfNeeded;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  InitHelper(rvh);
  InitHelper(rvhHeader);
  InitHelper(rvhFooter);
  rvh.RichView.SetHeader(rvhHeader.RichView.RVData);
  rvh.RichView.SetFooter(rvhFooter.RichView.RVData);
  MarginsMM := Rect(20,20,20,20);
  HeaderYMM := 10;
  FooterYMM := 10;
  PageSizeMM.cx := 210; // A4
  PageSizeMM.cy := 297; // A4
end;

procedure TForm1.mitOpenClick(Sender: TObject);
begin
  if OpenDialog1.Execute then begin
    rvh.RichView.Clear;
    rvhHeader.RichView.Clear;
    rvhFooter.RichView.Clear;
    rvh.RichView.DeleteUnusedStyles(True,True,True);
    rvh.RichView.LoadRTF(OpenDialog1.FileName);
    HeaderYMM := rvh.RichView.RTFReadProperties.HeaderYMM;
    FooterYMM := rvh.RichView.RTFReadProperties.FooterYMM;
    FillPageInfo;
    tbPages.Max := rvh.PagesCount;
    MakePageImage(tbPages.Position);
    Loaded := True;
    mitSave.Enabled := True;
  end;
end;

// Converting mm to pixels
function MMToPixels(mm, ppi: Integer): Integer;
begin
  Result := Round(mm*5*ppi/127);
end;

// creating metafile and metafile canvas
procedure TForm1.CreateMetafile(var wmf: TMetafile; var Canvas: TMetafileCanvas);
var DC: HDC;
begin
  wmf := TMetafile.Create;
  if UsePrinterResolution then begin
    DC := RV_GetPrinterDC;
    wmf.Width  := MMToPixels(PageSizeMM.cx, GetDeviceCaps(DC, LOGPIXELSX));
    wmf.Height := MMToPixels(PageSizeMM.cy, GetDeviceCaps(DC, LOGPIXELSY));
    end
  else begin
    DC := 0;
    wmf.Width  := MMToPixels(PageSizeMM.cx, Screen.PixelsPerInch);
    wmf.Height := MMToPixels(PageSizeMM.cy, Screen.PixelsPerInch);
  end;
  Canvas := TMetafileCanvas.Create(wmf, DC);
  if DC<>0 then begin
    Canvas.Font.PixelsPerInch := GetDeviceCaps(DC, LOGPIXELSY);
    DeleteDC(DC);
  end;
end;

// Filling PageInfo structure
procedure TForm1.FillPageInfo;
var wmf: TMetafile;
    Canvas: TMetafileCanvas;
begin
  CreateMetafile(wmf, Canvas);
  CalcPageLayout(Canvas, PageInfo);
  Canvas.Free;
  wmf.Free;
end;

// Creating metafile, drawing page image, assigning this metafule to Image1.Picture
procedure TForm1.MakePageImage(PageNo: Integer);
var wmf: TMetafile;
    Canvas: TMetafileCanvas;
begin
  CreateMetafile(wmf, Canvas);
  Canvas.Pen.Color := clBlack;
  Canvas.Brush.Color := clWhite;
  Canvas.Rectangle(0,0,PageInfo.PageSize.cx,PageInfo.PageSize.cy);
  Canvas.Brush.Style := bsClear;
  DrawPage(Canvas, PageNo, PageInfo);
  Canvas.Free;
  Image1.Picture.Graphic := wmf;
  wmf.Free;
end;

// Drawing page image on Canvas
procedure TForm1.DrawPage(Canvas: TCanvas; PageNo: Integer;
  const PageInfo: TPageInfo);
begin
  rvh.DrawPageAt(PageInfo.Doc.Left, PageInfo.Doc.Top, PageNo, Canvas, False,
    PageInfo.Doc.Bottom-PageInfo.Doc.Top);
  if rvhHeader.RichView.ItemCount>0 then
    rvhHeader.DrawPageAt(PageInfo.Header.Left, PageInfo.Header.Top, 1, Canvas, False,
      PageInfo.Header.Bottom-PageInfo.Header.Top);
  if rvhFooter.RichView.ItemCount>0 then
    rvhFooter.DrawPageAt(PageInfo.Footer.Left, PageInfo.Footer.Top, 1, Canvas, False,
      PageInfo.Footer.Bottom-PageInfo.Footer.Top);
end;


// Filling PageInfo - calculating size and position of header, footer and main doc
procedure TForm1.CalcPageLayout(Canvas: TCanvas; var PageInfo: TPageInfo);
var ppix,ppiy: Integer;
begin
  ppix := GetDeviceCaps(Canvas.Handle, LOGPIXELSX);
  ppiy := GetDeviceCaps(Canvas.Handle, LOGPIXELSY);

  PageInfo.PageSize.cx := MMToPixels(PageSizeMM.cx, ppix);
  PageInfo.PageSize.cy := MMToPixels(PageSizeMM.cy, ppiy);

  PageInfo.Header := Rect(-1,-1,-1,-1);
  PageInfo.Footer := Rect(-1,-1,-1,-1);

  PageInfo.Doc.Left  := MMToPixels(MarginsMM.Left, ppix);
  PageInfo.Doc.Right := PageInfo.PageSize.cx-MMToPixels(MarginsMM.Right, ppix);
  PageInfo.Doc.Top   := MMToPixels(MarginsMM.Top, ppiy);
  PageInfo.Doc.Bottom := PageInfo.PageSize.cy-MMToPixels(MarginsMM.Bottom, ppiy);

  if rvhHeader.RichView.ItemCount>0 then begin
    PageInfo.Header.Left  := PageInfo.Doc.Left;
    PageInfo.Header.Right := PageInfo.Doc.Right;
    rvhHeader.Init(Canvas, PageInfo.Header.Right-PageInfo.Header.Left);
    rvhHeader.FormatNextPage($FFFFFFF);
    PageInfo.Header.Top   := MMToPixels(HeaderYMM, ppiy);
    PageInfo.Header.Bottom := PageInfo.Header.Top+rvhHeader.GetLastPageHeight;
    if PageInfo.Header.Bottom>PageInfo.Doc.Top then
      PageInfo.Doc.Top := PageInfo.Header.Bottom;
  end;
  if rvhFooter.RichView.ItemCount>0 then begin
    PageInfo.Footer.Left  := PageInfo.Doc.Left;
    PageInfo.Footer.Right := PageInfo.Doc.Right;
    rvhFooter.Init(Canvas, PageInfo.Header.Right-PageInfo.Header.Left);
    rvhFooter.FormatNextPage($FFFFFFF);
    PageInfo.Footer.Bottom := PageInfo.PageSize.cy-MMToPixels(FooterYMM, ppiy);
    PageInfo.Footer.Top := PageInfo.Footer.Bottom-rvhFooter.GetLastPageHeight;
    if PageInfo.Footer.Top<PageInfo.Doc.Bottom then
      PageInfo.Doc.Bottom := PageInfo.Footer.Top;
  end;
  rvh.Init(Canvas, PageInfo.Doc.Right-PageInfo.Doc.Left);
  while rvh.FormatNextPage(PageInfo.Doc.Bottom-PageInfo.Doc.Top) do;
end;

// saving metafile (current page image) to file
procedure TForm1.mitSaveClick(Sender: TObject);
begin
  if SaveDialog1.Execute then
    Image1.Picture.SaveToFile(SaveDialog1.FileName);
end;

// creating metafile in screen resolution
procedure TForm1.mitScreenClick(Sender: TObject);
begin
  UsePrinterResolution := False;
  mitScreen.Checked    := True;
  mitPrinter.Checked   := False;
  if Loaded then begin
    FillPageInfo;
    MakePageImage(tbPages.Position);
  end;
end;

// creating metafile in printer resolution
procedure TForm1.mitPrinterClick(Sender: TObject);
begin
  UsePrinterResolution := True;
  mitScreen.Checked    := False;
  mitPrinter.Checked   := True;
  if Loaded then begin
    FillPageInfo;
    MakePageImage(tbPages.Position);
  end;
end;

// browsing pages
procedure TForm1.tbPagesChange(Sender: TObject);
begin
  if Loaded then
    MakePageImage(tbPages.Position);
end;

// switching view: 1:1 vs full page
procedure TForm1.mitAutoResizeClick(Sender: TObject);
begin
  mitAutoResize.Checked := not mitAutoResize.Checked;
  Image1.Proportional := mitAutoResize.Checked;
end;

end.
